from os import environ as env
envvar = 'PTS_TQ_VER'
if envvar not in env:
    tqshacl_ver = '1.4.2'
else:
    tqshacl_ver = env[envvar]
assert(len(tqshacl_ver.split('.')) == 3)


envvar = 'PTS_PREFER_SYSJAVA'
if envvar not in env:
    prefer_sysjava = True
else:
    _ = env[envvar].lower()
    assert(_ in {'true', 'false'})
    if _ == 'true':     prefer_sysjava = True
    if _ == 'false':    prefer_sysjava = False




from pathlib import Path
def printerrs(s):
    if (s.returncode != 0):
        print('ERRORS: process did not exit with 0')
        print(s.stderr)
    return s.stdout

def common(  cmd,
        data,
        shapes,
        out):
    data = Path(data)
    if shapes is not None: shapes = Path(shapes).as_posix()
    data = (data.as_posix())
    assert(cmd in {'infer', 'validate'})
    if cmd == 'infer': from     .run import infer       as f
    if cmd == 'validate': from  .run import validate    as f
    _ = f(data, shapes=shapes)
    rc = _.returncode
    _ = printerrs(_)
    if out is not None:
        open(out, 'w').write(_)
        return out
    else:
        return _

class defaults:
    data =      Path('data.ttl')
    shapes =    Path('shapes.ttl')
    # better than None bc stdout could be mixed with errors/warnings
    out =       Path('out.ttl')
def infer(
        data: Path      =defaults.data,
        shapes:Path     =defaults.shapes,
        out:Path | None =defaults.out):#Path('inferred.ttl')):
    return common('infer', data, shapes, out)
def validate(
        data: Path      =defaults.data,
        shapes:Path     =defaults.shapes,
        out:Path | None =defaults.out):#Path('inferred.ttl')):
    return common('validate', data, shapes, out)


from .run import cmd
from fire import Fire
Fire({f.__name__:f for f in {cmd, validate, infer}})
exit(0)



from pathlib import Path
from typing import Literal

def env():
    from os import environ
    from .topquadrant.install import Shacl
    si = Shacl()
    return {**environ,
        'SHACL_HOME': str(si.home),
        'SHACL_CP': f"{si.lib}/*", # need a star for some reason
        'LOGGING': str(si.logging),
          }

NOTSET = object()
def tryenv(k):
    # ugly
    try:
        return env()[k]
    except:
        return NOTSET

def cmd(
        cmd:Literal['validate']|Literal['infer'],
        datafile: Path,
        shapesfile: Path=None,
        shacl_cp=tryenv('SHACL_CP'), jvm_args='', logging=tryenv('LOGGING'),
        ):
    """command passed to java to run topquadrant shacl."""
    assert(cmd in {'validate', 'infer'})
    if (shacl_cp == NOTSET) or (logging == NOTSET):
        raise EnvironmentError("shacl_cp or logging not set")
    
    logging = f"-Dlog4j.configurationFile={logging}" if logging else ''
    # class path
    # quote so no funny shell parsing happens (on linux)
    shacl_cp = f"-cp \"{shacl_cp}\"" 
    cmd = cmd[0].upper()+cmd[1:]
    from .topquadrant.install import Java
    java = Java.get()
    cmd = f"{java} {jvm_args} {logging} {shacl_cp} org.topbraid.shacl.tools.{cmd}"
    _ = f"{cmd} -datafile {datafile} "
    if shapesfile:
        _ = _+f"-shapesfile {shapesfile}"
    return _

import logging
logger = logging.getLogger('topquadrant')
def check_proc_manually(cmd, proc):
    # further guard to fail
    # in case topquadrant does not exit with an error
    # that's why check is false below
    if any(w in proc.stderr.lower() for w in {'exception', 'error'}):
        from subprocess import CalledProcessError
        from sys import stderr
        print(proc.stderr, file=stderr)
        raise CalledProcessError(proc.returncode, cmd, stderr=proc.stderr)
    
    # filter out warnings to ensure valid ttl of stdout
    _ = []
    for l in proc.stdout.split('\n'):
        if ('warn' and 'riot') in l.lower():
            logger.warning(l)
        else:
            _.append(l)
    proc.stdout = '\n'.join(_)
    return proc


def common(cmdnm, data, shapes):
    c = cmd(cmdnm, data, shapes)
    from subprocess import run
    _ = run(
            c, check=False, env=env(), shell=True,
            capture_output=True, text=True )
    _ = check_proc_manually(c, _)
    return _

def validate(data: Path, *, shapes:Path|None=None):
    _ = common('validate', data, shapes)
    return _
def infer(data: Path, *, shapes:Path|None=None):
    _ = common('infer', data, shapes)
    return _



from pathlib import Path

class Java:
    @staticmethod
    def get_existing_java():
        import shutil
        path = shutil.which("java")
        return path
    
    from ..config import prefer_sysjava
    @classmethod
    def get(cls, use_existing=prefer_sysjava):
        _ = cls.get_existing_java()
        if _ and use_existing: return _
        else:
            return cls().bin

    class defaults:
        ver = '21'
        jre = True
    def __init__(self, ver: str=defaults.ver, jre: bool=defaults.jre):
        self.ver = ver
        self.jre = jre
        self.install(ver, jre)
    
    def install(self, ver: str, jre: bool):
        if not self.dir:
            import jdk
            print('installing java')
            jdk.install(ver, jre=jre)

    
    @property
    def base(self):
        if self.jre:    return Path.home() / '.jre'
        else:           return Path.home() / '.jdk'
    @property
    def dir(self):
        if not self.base.exists():
            return
        for d in self.base.iterdir():
            if d.is_dir():
                if f'jdk-{self.ver}' in str(d):
                    return d
    @property
    def bin(self):
        dir = self.dir
        assert(dir)
        dir = dir / 'bin'
        j = 'java'
        fns = [j, f'{j}.exe', f'{j}.sh', f'{j}.bat' ]
        for f in fns:
            _ = dir / f
            if _.exists(): return _
        raise ProcessLookupError('java not found')


from ..config import tqshacl_ver as ver
class Shacl:
    def __init__(self, ver=ver, overwrite=False) -> None:
        _ = Path(__file__).parent / 'src' # could go under java.home
        self.dir = self.download_shacl(ver, _ / f'shacl-{ver}' , overwrite=overwrite)
        gi = (_ / '.gitignore')
        if not gi.exists():
            gi.touch()
            # ignore everything
            open(gi, 'w').write('*')

        self.ver = ver
        assert(self.home.   exists())
        assert(self.logging.exists())
        assert(self.lib.    exists())

    @staticmethod
    def download_shacl(ver, dir, overwrite=False) -> Path:
        if dir.exists() and not overwrite:
            return dir
        
        import urllib.request
        _ = urllib.request.urlopen(
            ('https://repo1.maven.org/maven2/org/'
             'topbraid/shacl'
             f'/{ver}/shacl-{ver}-bin.zip'))
        _ = _.read()
        from zipfile import ZipFile
        from io import BytesIO
        _ = ZipFile(BytesIO(_))
        _.extractall(dir)
        return dir

    @property
    def home(self) -> Path:
        return self.dir / f"shacl-{self.ver}"
    @property
    def logging(self) -> Path:
        return self.home / "log4j2.properties"
    @property
    def lib(self) -> Path:
        return self.home / 'lib'

